#include <check.h>
#include <stdint.h>
#include "../fwlogs/parser.h"
// Note: data here are real live recorded packets with utility fwrecorder

void unittests_add_suite(Suite*);

static struct packet_data dt;

// This is connection with curl
static const uint8_t tcp_80_data[] = {
	0x60, 0x05, 0xe2, 0x4d, 0x00, 0x28, 0x06, 0x40, 0x20, 0x01, 0x1a, 0xe9,
	0x00, 0xa3, 0xdf, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01,
	0x20, 0x01, 0x1a, 0xe9, 0x00, 0xa3, 0xdf, 0x80, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x60, 0x9e, 0x60, 0x00, 0x50, 0xf9, 0xf8, 0x48, 0x15,
	0x00, 0x00, 0x00, 0x00, 0xa0, 0x02, 0x6f, 0xe0, 0x82, 0x2c, 0x00, 0x00,
	0x02, 0x04, 0x05, 0x98, 0x04, 0x02, 0x08, 0x0a, 0x81, 0x5f, 0xbd, 0x74,
	0x00, 0x00, 0x00, 0x00, 0x01, 0x03, 0x03, 0x07
};
static const size_t tcp_80_len =
	sizeof(tcp_80_data) / sizeof(*tcp_80_data);

static const uint8_t udp_53_data[] = {
	0x60, 0x05, 0x9b, 0x20, 0x00, 0x20, 0x11, 0x40, 0x20, 0x01, 0x1a, 0xe9,
	0x00, 0xa3, 0xdf, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01,
	0x20, 0x01, 0x1a, 0xe9, 0x00, 0xa3, 0xdf, 0x80, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x60, 0x88, 0x40, 0x00, 0x35, 0x00, 0x20, 0x4b, 0x11,
	0x0b, 0x73, 0x01, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x03, 0x6e, 0x69, 0x63, 0x02, 0x63, 0x7a, 0x00, 0x00, 0x01, 0x00, 0x01
};
static const size_t udp_53_len = sizeof(udp_53_data) / sizeof(*udp_53_data);

static const uint8_t icmp_data[] = {
	0x60, 0x01, 0x19, 0xf3, 0x00, 0x40, 0x3a, 0x40, 0x20, 0x01, 0x1a, 0xe9,
	0x00, 0xa3, 0xdf, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01,
	0x20, 0x01, 0x1a, 0xe9, 0x00, 0xa3, 0xdf, 0x80, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x60, 0x80, 0x00, 0xa2, 0xf9, 0x1d, 0xa4, 0x00, 0x01,
	0x8e, 0x69, 0x99, 0x60, 0x74, 0x9c, 0x01, 0x00, 0x08, 0x09, 0x0a, 0x0b,
	0x0c, 0x0d, 0x0e, 0x0f, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17,
	0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f, 0x20, 0x21, 0x22, 0x23,
	0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f,
	0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37
};
static const size_t icmp_len = sizeof(icmp_data) / sizeof(*icmp_data);


static struct packets {
	const uint8_t *data;
	size_t len;
	struct packet_data expected;
} packets[] = {
	{
		.data = tcp_80_data,
		.len = tcp_80_len,
		.expected = (struct packet_data){
			.proto = "TCP",
			.source_ip = "2001:1ae9:a3:df80::1",
			.dest_ip = "2001:1ae9:a3:df80::60",
			.source_port = 40544,
			.dest_port = 80,
		}
	},
	{
		.data = udp_53_data,
		.len = udp_53_len,
		.expected = (struct packet_data){
			.proto = "UDP",
			.source_ip = "2001:1ae9:a3:df80::1",
			.dest_ip = "2001:1ae9:a3:df80::60",
			.source_port = 34880,
			.dest_port = 53,
		}
	},
	{
		.data = icmp_data,
		.len = icmp_len,
		.expected = (struct packet_data){
			.proto = "ICMPV6",
			.source_ip = "2001:1ae9:a3:df80::1",
			.dest_ip = "2001:1ae9:a3:df80::60",
			.source_port = 0,
			.dest_port = 0,
		}
	},
};


// This is connection with curl to port 80
START_TEST(ipv6_parse) {
	ck_assert(parse_packet(packets[_i].data, packets[_i].len, &dt));

	// We ignore time as we can't match it exactly anyway
	ck_assert_str_eq(dt.proto, packets[_i].expected.proto);
	ck_assert_str_eq(dt.source_ip, packets[_i].expected.source_ip);
	ck_assert_str_eq(dt.dest_ip, packets[_i].expected.dest_ip);
	ck_assert_int_eq(dt.source_port, packets[_i].expected.source_port);
	ck_assert_int_eq(dt.dest_port, packets[_i].expected.dest_port);
}
END_TEST


// We are using truncated tcp_80_data in these invalid packets but in general it
// should not matter what data we pass. The important part is data length.
static const uint8_t invalid_ip_data[] = {
	0x60, 0x05, 0xe2, 0x4d, 0x00, 0x28, 0x06, 0x40, 0x20, 0x01, 0x1a, 0xe9,
	0x00, 0xa3, 0xdf, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01,
};
static const size_t invalid_ip_len =
	sizeof(invalid_ip_data) / sizeof(*invalid_ip_data);

START_TEST(invalid_ip) {
	ck_assert(!parse_packet(invalid_ip_data, invalid_ip_len, &dt));
}
END_TEST

static const uint8_t invalid_tcp_data[] = {
	0x60, 0x05, 0xe2, 0x4d, 0x00, 0x28, 0x06, 0x40, 0x20, 0x01, 0x1a, 0xe9,
	0x00, 0xa3, 0xdf, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01,
	0x20, 0x01, 0x1a, 0xe9, 0x00, 0xa3, 0xdf, 0x80, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x60, 0x9e, 0x60, 0x00, 0x50, 0xf9, 0xf8, 0x48, 0x15,
};
static const size_t invalid_tcp_len =
	sizeof(invalid_tcp_data) / sizeof(*invalid_tcp_data);

START_TEST(invalid_tcp) {
	ck_assert(!parse_packet(invalid_tcp_data, invalid_tcp_len, &dt));
}
END_TEST



__attribute__((constructor))
static void suite() {
	Suite *suite = suite_create("parse ipv6");

	TCase *valid_case = tcase_create("valid");
	tcase_add_loop_test(valid_case, ipv6_parse, 0,
		sizeof(packets) / sizeof(struct packets));
	suite_add_tcase(suite, valid_case);

	TCase *invalid_case = tcase_create("invalid");
	tcase_add_test(invalid_case, invalid_ip);
	tcase_add_test(invalid_case, invalid_tcp);
	suite_add_tcase(suite, invalid_case);

	unittests_add_suite(suite);
}
